@@ -5,7 +5,6 @@ import gi
 
 gi.require_version("Gtk", "3.0")
 from gi.repository import GLib, Gtk, Pango
-from contextlib import suppress
 from math import pi, sqrt, trunc
 from statistics import median
 from time import time
@@ -16,8 +15,7 @@ from ks_includes.KlippyGtk import find_widget
 class Panel(ScreenPanel):
     def __init__(self, screen, title):
         super().__init__(screen, title)
-        self.grid = self._gtk.HomogeneousGrid()
-        self.grid.set_row_homogeneous(False)
+        self.grid = Gtk.Grid(column_homogeneous=True)
         self.pos_z = 0.0
         self.extrusion = 100
         self.speed_factor = 1.0
@@ -47,9 +45,7 @@ class Panel(ScreenPanel):
                 'flowrate']
 
         for item in data:
-            self.labels[item] = Gtk.Label(label="-")
-            self.labels[item].set_vexpand(True)
-            self.labels[item].set_hexpand(True)
+            self.labels[item] = Gtk.Label(label="-", hexpand=True, vexpand=True)
 
         self.labels['left'] = Gtk.Label(_("Left:"))
         self.labels['elapsed'] = Gtk.Label(_("Elapsed:"))
@@ -83,9 +79,8 @@ class Panel(ScreenPanel):
                 "speed": "-"
             }
 
-        self.labels['file'] = Gtk.Label(label="Filename")
+        self.labels['file'] = Gtk.Label(label="Filename", hexpand=True)
         self.labels['file'].get_style_context().add_class("printing-filename")
-        self.labels['file'].set_hexpand(True)
         self.labels['status'] = Gtk.Label(label="Status")
         self.labels['status'].get_style_context().add_class("printing-status")
         self.labels['lcdmessage'] = Gtk.Label()
@@ -104,14 +99,12 @@ class Panel(ScreenPanel):
         self.labels['darea'] = Gtk.DrawingArea()
         self.labels['darea'].connect("draw", self.on_draw)
 
-        box = Gtk.Box()
-        box.set_halign(Gtk.Align.CENTER)
+        box = Gtk.Box(halign=Gtk.Align.CENTER)
         self.labels['progress_text'] = Gtk.Label(label="0%")
         self.labels['progress_text'].get_style_context().add_class("printing-progress-text")
         box.add(self.labels['progress_text'])
 
-        overlay = Gtk.Overlay()
-        overlay.set_hexpand(True)
+        overlay = Gtk.Overlay(hexpand=True)
         overlay.add(self.labels['darea'])
         overlay.add_overlay(box)
         self.grid.attach(overlay, 0, 0, 1, 1)
@@ -128,8 +121,7 @@ class Panel(ScreenPanel):
 
         self.buttons = {}
         self.create_buttons()
-        self.buttons['button_grid'] = self._gtk.HomogeneousGrid()
-        self.buttons['button_grid'].set_vexpand(False)
+        self.buttons['button_grid'] = Gtk.Grid(row_homogeneous=True, column_homogeneous=True, vexpand=False)
         self.grid.attach(self.buttons['button_grid'], 0, 3, 4, 1)
 
         self.create_status_grid()
@@ -154,77 +146,73 @@ class Panel(ScreenPanel):
         buttons['fan'].connect("clicked", self.menu_item_clicked, {"panel": "fan", "name": _("Fan")})
         self.buttons.update(buttons)
 
+        self.buttons['extruder'] = {}
+        for i, extruder in enumerate(self._printer.get_tools()):
+            self.labels[extruder] = Gtk.Label(label="-")
+            self.buttons['extruder'][extruder] = self._gtk.Button(f"extruder-{i}", "", None, self.bts,
+                                                                  Gtk.PositionType.LEFT, 1)
+            self.buttons['extruder'][extruder].set_label(self.labels[extruder].get_text())
+            self.buttons['extruder'][extruder].connect("clicked", self.menu_item_clicked,
+                                                       {"panel": "temperature", "name": _("Temperature"),
+                                                        'extra': extruder})
+            self.buttons['extruder'][extruder].set_halign(Gtk.Align.START)
+
         self.labels['temp_grid'] = Gtk.Grid()
         nlimit = 2 if self._screen.width <= 500 else 3
         n = 0
-        self.buttons['extruder'] = {}
-        self.current_extruder = self._printer.get_stat("toolhead", "extruder")
-        if self.current_extruder:
-            for i, extruder in enumerate(self._printer.get_tools()):
-                self.labels[extruder] = Gtk.Label(label="-")
-                self.buttons['extruder'][extruder] = self._gtk.Button(f"extruder-{i}", "", None, self.bts,
-                                                                      Gtk.PositionType.LEFT, 1)
-                self.buttons['extruder'][extruder].set_label(self.labels[extruder].get_text())
-                self.buttons['extruder'][extruder].connect("clicked", self.menu_item_clicked,
-                                                           {"panel": "temperature", "name": _("Temperature"),
-                                                            'extra': self.current_extruder})
-                self.buttons['extruder'][extruder].set_halign(Gtk.Align.START)
-            self.labels['temp_grid'].attach(self.buttons['extruder'][self.current_extruder], n, 0, 1, 1)
-            n += 1
+        if nlimit > 2 and len(self._printer.get_tools()) == 2:
+            for extruder in self.buttons['extruder']:
+                self.labels['temp_grid'].attach(self.buttons['extruder'][extruder], n, 0, 1, 1)
+                n += 1
+        else:
+            self.current_extruder = self._printer.get_stat("toolhead", "extruder")
+            if self.current_extruder:
+                self.labels['temp_grid'].attach(self.buttons['extruder'][self.current_extruder], n, 0, 1, 1)
+                n += 1
         self.buttons['heater'] = {}
         for dev in self._printer.get_heaters():
             if n >= nlimit:
                 break
             if dev == "heater_bed":
-                self.buttons['heater']['heater_bed'] = self._gtk.Button("bed", "", None, self.bts,
-                                                                        Gtk.PositionType.LEFT, 1)
-                self.labels['heater_bed'] = Gtk.Label(label="-")
-                self.buttons['heater']['heater_bed'].set_label(self.labels['heater_bed'].get_text())
-                self.buttons['heater']['heater_bed'].connect("clicked", self.menu_item_clicked,
-                                                             {"panel": "temperature", "name": _("Temperature"),
-                                                              'extra': 'heater_bed'})
-                self.buttons['heater']['heater_bed'].set_halign(Gtk.Align.START)
-                self.labels['temp_grid'].attach(self.buttons['heater']['heater_bed'], n, 0, 1, 1)
-                n += 1
-            elif dev.startswith("heater_generic"):
+                self.buttons['heater'][dev] = self._gtk.Button("bed", "", None, self.bts, Gtk.PositionType.LEFT, 1)
+            else:
                 self.buttons['heater'][dev] = self._gtk.Button("heater", "", None, self.bts, Gtk.PositionType.LEFT, 1)
-                self.labels[dev] = Gtk.Label(label="-")
-                self.buttons['heater'][dev].set_label(self.labels[dev].get_text())
-                self.buttons['heater'][dev].connect("clicked", self.menu_item_clicked,
-                                                    {"panel": "temperature", "name": _("Temperature"), "extra": dev})
-                self.buttons['heater'][dev].set_halign(Gtk.Align.START)
-                self.labels['temp_grid'].attach(self.buttons['heater'][dev], n, 0, 1, 1)
-                n += 1
+            self.labels[dev] = Gtk.Label(label="-")
+
+            self.buttons['heater'][dev].set_label(self.labels[dev].get_text())
+            self.buttons['heater'][dev].connect("clicked", self.menu_item_clicked,
+                                                {"panel": "temperature", "name": _("Temperature"), "extra": dev})
+            self.buttons['heater'][dev].set_halign(Gtk.Align.START)
+            self.labels['temp_grid'].attach(self.buttons['heater'][dev], n, 0, 1, 1)
+            n += 1
         extra_item = not self._show_heater_power
         if self.ks_printer_cfg is not None:
             titlebar_items = self.ks_printer_cfg.get("titlebar_items", "")
             if titlebar_items is not None:
                 titlebar_items = [str(i.strip()) for i in titlebar_items.split(',')]
                 logging.info(f"Titlebar items: {titlebar_items}")
-                for device in self._printer.get_heaters():
-                    if device.startswith("temperature_sensor"):
-                        name = " ".join(device.split(" ")[1:])
-                        for item in titlebar_items:
-                            if name == item:
-                                if extra_item:
-                                    extra_item = False
-                                    nlimit += 1
-                                if n >= nlimit:
-                                    break
-                                self.buttons['heater'][device] = self._gtk.Button("heat-up", "", None, self.bts,
-                                                                                  Gtk.PositionType.LEFT, 1)
-                                self.labels[device] = Gtk.Label(label="-")
-                                self.buttons['heater'][device].set_label(self.labels[device].get_text())
-                                self.buttons['heater'][device].connect("clicked", self.menu_item_clicked,
-                                                                       {"panel": "temperature",
-                                                                        "name": _("Temperature")})
-                                self.buttons['heater'][device].set_halign(Gtk.Align.START)
-                                self.labels['temp_grid'].attach(self.buttons['heater'][device], n, 0, 1, 1)
-                                n += 1
+                for device in self._printer.get_temp_sensors():
+                    name = " ".join(device.split(" ")[1:])
+                    for item in titlebar_items:
+                        if name == item:
+                            if extra_item:
+                                extra_item = False
+                                nlimit += 1
+                            if n >= nlimit:
                                 break
+                            self.buttons['heater'][device] = self._gtk.Button("heat-up", "", None, self.bts,
+                                                                              Gtk.PositionType.LEFT, 1)
+                            self.labels[device] = Gtk.Label(label="-")
+                            self.buttons['heater'][device].set_label(self.labels[device].get_text())
+                            self.buttons['heater'][device].connect("clicked", self.menu_item_clicked,
+                                                                   {"panel": "temperature",
+                                                                    "name": _("Temperature")})
+                            self.buttons['heater'][device].set_halign(Gtk.Align.START)
+                            self.labels['temp_grid'].attach(self.buttons['heater'][device], n, 0, 1, 1)
+                            n += 1
+                            break
 
-        szfe = Gtk.Grid()
-        szfe.set_column_homogeneous(True)
+        szfe = Gtk.Grid(column_homogeneous=True)
         szfe.attach(self.buttons['speed'], 0, 0, 3, 1)
         szfe.attach(self.buttons['z'], 2, 0, 2, 1)
         if self._printer.get_tools():
@@ -232,8 +220,7 @@ class Panel(ScreenPanel):
         if self._printer.get_fans():
             szfe.attach(self.buttons['fan'], 2, 1, 2, 1)
 
-        info = Gtk.Grid()
-        info.set_row_homogeneous(True)
+        info = Gtk.Grid(row_homogeneous=True)
         info.get_style_context().add_class("printing-info")
         info.attach(self.labels['temp_grid'], 0, 0, 1, 1)
         info.attach(szfe, 0, 1, 1, 2)
@@ -247,10 +234,7 @@ class Panel(ScreenPanel):
         goback.set_hexpand(False)
         goback.get_style_context().add_class("printing-info")
 
-        info = Gtk.Grid()
-        info.set_hexpand(True)
-        info.set_vexpand(True)
-        info.set_halign(Gtk.Align.START)
+        info = Gtk.Grid(hexpand=True, vexpand=True, halign=Gtk.Align.START)
         info.get_style_context().add_class("printing-info-secondary")
         info.attach(goback, 0, 0, 1, 6)
         info.attach(self.labels['flow'], 1, 0, 1, 1)
@@ -277,10 +261,7 @@ class Panel(ScreenPanel):
         pos_box.add(self.labels['pos_y'])
         pos_box.add(self.labels['pos_z'])
 
-        info = Gtk.Grid()
-        info.set_hexpand(True)
-        info.set_vexpand(True)
-        info.set_halign(Gtk.Align.START)
+        info = Gtk.Grid(hexpand=True, vexpand=True, halign=Gtk.Align.START)
         info.get_style_context().add_class("printing-info-secondary")
         info.attach(goback, 0, 0, 1, 6)
         info.attach(self.labels['speed_lbl'], 1, 0, 1, 1)
@@ -356,6 +337,7 @@ class Panel(ScreenPanel):
         if self.flow_timeout is not None:
             GLib.source_remove(self.flow_timeout)
             self.flow_timeout = None
+        self._files.remove_callback(self._callback_metadata)
 
     def create_buttons(self):
 
@@ -383,7 +365,7 @@ class Panel(ScreenPanel):
 
     def save_offset(self, widget, device):
         sign = "+" if self.zoffset > 0 else "-"
-        label = Gtk.Label()
+        label = Gtk.Label(hexpand=True, vexpand=True, wrap=True)
         if device == "probe":
             probe = self._printer.get_probe()
             saved_z_offset = probe['z_offset'] if probe else "?"
@@ -392,20 +374,12 @@ class Panel(ScreenPanel):
                             + _("Saved offset: %s") % saved_z_offset)
         elif device == "endstop":
             label.set_label(_("Apply %s%.3f offset to Endstop?") % (sign, abs(self.zoffset)))
-        label.set_hexpand(True)
-        label.set_halign(Gtk.Align.CENTER)
-        label.set_vexpand(True)
-        label.set_valign(Gtk.Align.CENTER)
-        label.set_line_wrap(True)
-        label.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
 
-        grid = self._gtk.HomogeneousGrid()
-        grid.attach(label, 0, 0, 1, 1)
         buttons = [
-            {"name": _("Apply"), "response": Gtk.ResponseType.APPLY},
-            {"name": _("Cancel"), "response": Gtk.ResponseType.CANCEL}
+            {"name": _("Apply"), "response": Gtk.ResponseType.APPLY, "style": 'dialog-default'},
+            {"name": _("Cancel"), "response": Gtk.ResponseType.CANCEL, "style": 'dialog-error'}
         ]
-        self._gtk.Dialog(_("Save Z"), buttons, grid, self.save_confirm, device)
+        self._gtk.Dialog(_("Save Z"), buttons, label, self.save_confirm, device)
 
     def save_confirm(self, dialog, response_id, device):
         self._gtk.remove_dialog(dialog)
@@ -438,19 +412,13 @@ class Panel(ScreenPanel):
 
     def cancel(self, widget):
         buttons = [
-            {"name": _("Cancel Print"), "response": Gtk.ResponseType.OK},
-            {"name": _("Go Back"), "response": Gtk.ResponseType.CANCEL}
+            {"name": _("Cancel Print"), "response": Gtk.ResponseType.OK, "style": 'dialog-error'},
+            {"name": _("Go Back"), "response": Gtk.ResponseType.CANCEL, "style": 'dialog-info'}
         ]
         if len(self._printer.get_stat("exclude_object", "objects")) > 1:
             buttons.insert(0, {"name": _("Exclude Object"), "response": Gtk.ResponseType.APPLY})
-        label = Gtk.Label()
+        label = Gtk.Label(hexpand=True, vexpand=True, wrap=True)
         label.set_markup(_("Are you sure you wish to cancel this print?"))
-        label.set_hexpand(True)
-        label.set_halign(Gtk.Align.CENTER)
-        label.set_vexpand(True)
-        label.set_valign(Gtk.Align.CENTER)
-        label.set_line_wrap(True)
-        label.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
         self._gtk.Dialog(_("Cancel"), buttons, label, self.cancel_confirm)
 
     def cancel_confirm(self, dialog, response_id):
@@ -479,13 +447,17 @@ class Panel(ScreenPanel):
         for arg in args:
             self.buttons[arg].set_sensitive(False)
 
-    def _callback_metadata(self, newfiles, deletedfiles, modifiedfiles):
-        if not bool(self.file_metadata) and self.filename in modifiedfiles:
+    def _callback_metadata(self, action, item):
+        if action == "update_metadata" and self.filename in item:
+            logging.info("Callback complete")
             self.update_file_metadata()
-            self._files.remove_file_callback(self._callback_metadata)
+            self._files.remove_callback(self._callback_metadata)
 
     def new_print(self):
         self._screen.close_screensaver()
+        if "virtual_sdcard" in self._printer.data:
+            logging.info("reseting progress")
+            self._printer.data["virtual_sdcard"]["progress"] = 0
         self.update_progress(0.0)
 
     def process_update(self, action, data):
@@ -518,41 +490,41 @@ class Panel(ScreenPanel):
                 f"{data['display_status']['message'] if data['display_status']['message'] is not None else ''}"
             )
 
-        with suppress(KeyError):
-            if data["toolhead"]["extruder"] != self.current_extruder:
+        if 'toolhead' in data:
+            if 'extruder' in data['toolhead'] and data['toolhead']['extruder'] != self.current_extruder:
                 self.labels['temp_grid'].remove_column(0)
                 self.labels['temp_grid'].insert_column(0)
                 self.current_extruder = data["toolhead"]["extruder"]
                 self.labels['temp_grid'].attach(self.buttons['extruder'][self.current_extruder], 0, 0, 1, 1)
                 self._screen.show_all()
-        with suppress(KeyError):
-            self.labels['max_accel'].set_label(f"{data['toolhead']['max_accel']:.0f} {self.mms2}")
-        with suppress(KeyError):
+            if "max_accel" in data["toolhead"]:
+                self.labels['max_accel'].set_label(f"{data['toolhead']['max_accel']:.0f} {self.mms2}")
+        if 'extruder' in data and 'pressure_advance' in data['extruder']:
             self.labels['advance'].set_label(f"{data['extruder']['pressure_advance']:.2f}")
 
-        if "gcode_move" in data:
-            with suppress(KeyError):
+        if 'gcode_move' in data:
+            if 'gcode_position' in data['gcode_move']:
                 self.pos_z = round(float(data['gcode_move']['gcode_position'][2]), 2)
                 self.buttons['z'].set_label(f"Z: {self.pos_z:6.2f}{f'/{self.oheight}' if self.oheight > 0 else ''}")
-            with suppress(KeyError):
-                self.extrusion = round(float(data["gcode_move"]["extrude_factor"]) * 100)
+            if 'extrude_factor' in data['gcode_move']:
+                self.extrusion = round(float(data['gcode_move']['extrude_factor']) * 100)
                 self.labels['extrude_factor'].set_label(f"{self.extrusion:3}%")
-            with suppress(KeyError):
-                self.speed = round(float(data["gcode_move"]["speed_factor"]) * 100)
-                self.speed_factor = float(data["gcode_move"]["speed_factor"])
+            if 'speed_factor' in data['gcode_move']:
+                self.speed = round(float(data['gcode_move']['speed_factor']) * 100)
+                self.speed_factor = float(data['gcode_move']['speed_factor'])
                 self.labels['speed_factor'].set_label(f"{self.speed:3}%")
-            with suppress(KeyError):
+            if 'speed' in data['gcode_move']:
                 self.req_speed = round(float(data["gcode_move"]["speed"]) / 60 * self.speed_factor)
                 self.labels['req_speed'].set_label(
                     f"{self.speed}% {self.vel:3.0f}/{self.req_speed:3.0f} "
                     f"{f'{self.mms}' if self.vel < 1000 and self.req_speed < 1000 and self._screen.width > 500 else ''}"
                 )
                 self.buttons['speed'].set_label(self.labels['req_speed'].get_label())
-            with suppress(KeyError):
-                self.zoffset = float(data["gcode_move"]["homing_origin"][2])
+            if 'homing_origin' in data['gcode_move']:
+                self.zoffset = float(data['gcode_move']['homing_origin'][2])
                 self.labels['zoffset'].set_label(f"{self.zoffset:.3f} {self.mm}")
-        if "motion_report" in data:
-            with suppress(KeyError):
+        if 'motion_report' in data:
+            if 'live_position' in data['motion_report']:
                 self.labels['pos_x'].set_label(f"X: {data['motion_report']['live_position'][0]:6.2f}")
                 self.labels['pos_y'].set_label(f"Y: {data['motion_report']['live_position'][1]:6.2f}")
                 self.labels['pos_z'].set_label(f"Z: {data['motion_report']['live_position'][2]:6.2f}")
@@ -564,14 +536,14 @@ class Panel(ScreenPanel):
                     evelocity = (pos[3] - self.prev_pos[0][3]) / interval
                     self.flowstore.append(self.fila_section * evelocity)
                 self.prev_pos = [pos, now]
-            with suppress(KeyError):
+            if 'live_velocity' in data['motion_report']:
                 self.vel = float(data["motion_report"]["live_velocity"])
                 self.labels['req_speed'].set_label(
                     f"{self.speed}% {self.vel:3.0f}/{self.req_speed:3.0f} "
                     f"{f'{self.mms}' if self.vel < 1000 and self.req_speed < 1000 and self._screen.width > 500 else ''}"
                 )
                 self.buttons['speed'].set_label(self.labels['req_speed'].get_label())
-            with suppress(KeyError):
+            if 'live_extruder_velocity' in data['motion_report']:
                 self.flowstore.append(self.fila_section * float(data["motion_report"]["live_extruder_velocity"]))
         fan_label = ""
         for fan in self.fans:
@@ -580,28 +552,27 @@ class Panel(ScreenPanel):
         if fan_label:
             self.buttons['fan'].set_label(fan_label[:12])
         if "print_stats" in data:
-            with suppress(KeyError):
+            if 'state' in data['print_stats']:
                 self.set_state(
                     data["print_stats"]["state"],
                     msg=f'{data["print_stats"]["message"] if "message" in data["print_stats"] else ""}'
                 )
-            with suppress(KeyError):
+            if 'filename' in data['print_stats']:
                 self.update_filename(data['print_stats']["filename"])
-            with suppress(KeyError):
-                if 'filament_used' in data["print_stats"]:
-                    self.labels['filament_used'].set_label(
-                        f"{float(data['print_stats']['filament_used']) / 1000:.1f} m"
-                    )
+            if 'filament_used' in data['print_stats']:
+                self.labels['filament_used'].set_label(
+                    f"{float(data['print_stats']['filament_used']) / 1000:.1f} m"
+                )
             if 'info' in data["print_stats"]:
-                with suppress(KeyError):
-                    if data["print_stats"]['info']['total_layer'] is not None:
-                        self.labels['total_layers'].set_label(f"{data['print_stats']['info']['total_layer']}")
-                with suppress(KeyError):
-                    if data["print_stats"]['info']['current_layer'] is not None:
-                        self.labels['layer'].set_label(
-                            f"{data['print_stats']['info']['current_layer']} / "
-                            f"{self.labels['total_layers'].get_text()}"
-                        )
+                if ('total_layer' in data['print_stats']['info']
+                        and data["print_stats"]['info']['total_layer'] is not None):
+                    self.labels['total_layers'].set_label(f"{data['print_stats']['info']['total_layer']}")
+                if ('current_layer' in data['print_stats']['info']
+                        and data['print_stats']['info']['current_layer'] is not None):
+                    self.labels['layer'].set_label(
+                        f"{data['print_stats']['info']['current_layer']} / "
+                        f"{self.labels['total_layers'].get_text()}"
+                    )
             elif "layer_height" in self.file_metadata and "object_height" in self.file_metadata:
                 self.labels['layer'].set_label(
                     f"{1 + round((self.pos_z - self.f_layer_h) / self.layer_h)} / "
@@ -622,63 +593,60 @@ class Panel(ScreenPanel):
     def update_time_left(self):
         total_duration = float(self._printer.get_stat('print_stats', 'total_duration'))
         print_duration = float(self._printer.get_stat('print_stats', 'print_duration'))
-        if 'filament_total' in self.file_metadata and not self.file_metadata['filament_total']:  # No-extrusion
+        if not self.file_metadata.get('filament_total'):  # No-extrusion
             print_duration = total_duration
         fila_used = float(self._printer.get_stat('print_stats', 'filament_used'))
-        progress = float(self._printer.get_stat("virtual_sdcard", "progress"))
+        if "gcode_start_byte" in self.file_metadata:
+            progress = (max(self._printer.get_stat('virtual_sdcard', 'file_position') -
+                        self.file_metadata['gcode_start_byte'], 0) / (self.file_metadata['gcode_end_byte'] -
+                        self.file_metadata['gcode_start_byte']))
+        else:
+            progress = self._printer.get_stat('virtual_sdcard', 'progress')
         self.labels["duration"].set_label(self.format_time(total_duration))
         elapsed_label = f"{self.labels['elapsed'].get_text()}  {self.labels['duration'].get_text()}"
         self.buttons['elapsed'].set_label(elapsed_label)
-        estimated = 0
-        slicer_time = filament_time = file_time = None
+        estimated = slicer_time = filament_time = file_time = 0
         timeleft_type = self._config.get_config()['main'].get('print_estimate_method', 'auto')
 
-        with suppress(KeyError):
-            if self.file_metadata['estimated_time'] > 0:
-                # speed_factor compensation based on empirical testing
-                spdcomp = sqrt(self.speed_factor)
-                slicer_time = ((self.file_metadata['estimated_time']) / spdcomp)
-        self.labels["slicer_time"].set_label(self.format_time(slicer_time))
+        if 'estimated_time' in self.file_metadata and self.file_metadata['estimated_time'] > 1:
+            spdcomp = sqrt(self.speed_factor)
+            slicer_time = ((self.file_metadata['estimated_time']) / spdcomp)
+            self.labels["slicer_time"].set_label(self.format_time(slicer_time))
 
-        with suppress(Exception):
-            if self.file_metadata['filament_total'] > fila_used:
-                filament_time = (print_duration / (fila_used / self.file_metadata['filament_total']))
-        self.labels["filament_time"].set_label(self.format_time(filament_time))
-        with suppress(ZeroDivisionError):
+        if 'filament_total' in self.file_metadata and self.file_metadata['filament_total'] >= fila_used > 0:
+            filament_time = (print_duration / (fila_used / self.file_metadata['filament_total']))
+            self.labels["filament_time"].set_label(self.format_time(filament_time))
+        if progress > 0:
             file_time = (print_duration / progress)
-        self.labels["file_time"].set_label(self.format_time(file_time))
+            self.labels["file_time"].set_label(self.format_time(file_time))
 
         if timeleft_type == "file":
             estimated = file_time
         elif timeleft_type == "filament":
             estimated = filament_time
-        elif slicer_time is not None:
-            if timeleft_type == "slicer":
-                estimated = slicer_time
-            elif filament_time is not None and progress > 0.14:
-                # Weighted arithmetic mean (Slicer is the most accurate)
-                estimated = (slicer_time * 3 + filament_time + file_time) / 5
-            else:
-                # At the begining file and filament are innacurate
-                estimated = slicer_time
-
-        if estimated < 1:
-            if file_time is None:
-                return
-            elif filament_time is not None and filament_time > 1:
+        elif timeleft_type == "slicer":
+            estimated = slicer_time
+        elif estimated < 1:  # Auto
+            if slicer_time > 1:
+                if progress < 0.15:
+                    # At the begining file and filament are innacurate
+                    estimated = slicer_time
+                elif filament_time > 1 and file_time > 1:
+                    # Weighted arithmetic mean (Slicer is the most accurate)
+                    estimated = (slicer_time * 3 + filament_time + file_time) / 5
+                elif file_time > 1:
+                    # Weighted arithmetic mean (Slicer is the most accurate)
+                    estimated = (slicer_time * 2 + file_time) / 3
+            elif filament_time > 1 and file_time > 1:
                 estimated = (filament_time + file_time) / 2
-            estimated = file_time
-        if estimated < 1:
-            return
-        progress = min(max(print_duration / estimated, 0), 1)
-        if progress >= 1:
-            estimated = file_time
+            elif file_time > 1:
+                estimated = file_time
+        if estimated > 1:
             progress = min(max(print_duration / estimated, 0), 1)
-
-        self.labels["est_time"].set_label(self.format_time(estimated))
-        self.labels["time_left"].set_label(self.format_eta(estimated, print_duration))
-        remaining_label = f"{self.labels['left'].get_text()}  {self.labels['time_left'].get_text()}"
-        self.buttons['left'].set_label(remaining_label)
+            self.labels["est_time"].set_label(self.format_time(estimated))
+            self.labels["time_left"].set_label(self.format_eta(estimated, print_duration))
+            remaining_label = f"{self.labels['left'].get_text()}  {self.labels['time_left'].get_text()}"
+            self.buttons['left'].set_label(remaining_label)
         self.update_progress(progress)
 
     def update_progress(self, progress: float):
@@ -780,15 +748,12 @@ class Panel(ScreenPanel):
         buttons = [
             {"name": _("Close"), "response": Gtk.ResponseType.CANCEL}
         ]
-        height = self._screen.height * .9 - self._gtk.font_size * 7.5
-        pixbuf = self.get_file_image(self.filename, self._screen.width * .9, height)
+        pixbuf = self.get_file_image(self.filename, self._screen.width * .9, self._screen.height * .75)
         if pixbuf is None:
             return
         image = Gtk.Image.new_from_pixbuf(pixbuf)
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        box.add(image)
-        box.set_vexpand(True)
-        self._gtk.Dialog(self.filename, buttons, box, self.close_fullscreen_thumbnail)
+        image.set_vexpand(True)
+        self._gtk.Dialog(self.filename, buttons, image, self.close_fullscreen_thumbnail)
 
     def close_fullscreen_thumbnail(self, dialog, response_id):
         self._gtk.remove_dialog(dialog)
@@ -838,7 +803,7 @@ class Panel(ScreenPanel):
             if "filament_total" in self.file_metadata:
                 self.labels['filament_total'].set_label(f"{float(self.file_metadata['filament_total']) / 1000:.1f} m")
         else:
-            self.file_metadata = {}
             logging.debug("Cannot find file metadata. Listening for updated metadata")
-            self._screen.files.add_file_callback(self._callback_metadata)
+            self._screen.files.add_callback(self._callback_metadata)
+            self._files.request_metadata(self.filename)
         self.show_file_thumbnail()
