@@ -69,21 +69,14 @@ class Panel(ScreenPanel):
     def add_option(self, option, optname, units, value):
         logging.info(f"Adding option: {option}")
 
-        name = Gtk.Label()
+        name = Gtk.Label(hexpand=True, vexpand=True, halign=Gtk.Align.START, valign=Gtk.Align.CENTER,
+                         wrap=True, wrap_mode=Pango.WrapMode.WORD_CHAR)
         name.set_markup(f"<big><b>{optname}</b></big> ({units})")
-        name.set_hexpand(True)
-        name.set_vexpand(True)
-        name.set_halign(Gtk.Align.START)
-        name.set_valign(Gtk.Align.CENTER)
-        name.set_line_wrap(True)
-        name.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
 
         # adj (value, lower, upper, step_increment, page_increment, page_size)
         adj = Gtk.Adjustment(value, 1, (value * 1.5), 1, 5, 0)
-        scale = Gtk.Scale.new(orientation=Gtk.Orientation.HORIZONTAL, adjustment=adj)
-        scale.set_digits(0)
-        scale.set_hexpand(True)
-        scale.set_has_origin(True)
+        scale = Gtk.Scale(adjustment=adj,
+                          digits=0, hexpand=True, has_origin=True)
         scale.get_style_context().add_class("option_slider")
         scale.connect("button-release-event", self.set_opt_value, option)
         self.values[option] = value
