@@ -1,6 +1,7 @@
 #!/usr/bin/python
 
 import argparse
+import gc
 import json
 import logging
 import os
@@ -25,6 +26,7 @@ from ks_includes.files import KlippyFiles
 from ks_includes.KlippyGtk import KlippyGtk
 from ks_includes.printer import Printer
 from ks_includes.widgets.keyboard import Keyboard
+from ks_includes.widgets.prompts import Prompt
 from ks_includes.config import KlipperScreenConfig
 from panels.base_panel import BasePanel
 
@@ -95,6 +97,7 @@ class KlipperScreen(Gtk.Window):
     wayland = False
     windowed = False
     notification_log = []
+    prompt = None
 
     def __init__(self, args):
         try:
@@ -102,12 +105,14 @@ class KlipperScreen(Gtk.Window):
         except Exception as e:
             logging.exception(f"{e}\n\n{traceback.format_exc()}")
             raise RuntimeError from e
+        GLib.set_prgname('KlipperScreen')
         self.blanking_time = 600
         self.use_dpms = True
         self.apiclient = None
         self.dialogs = []
         self.confirm = None
         self.panels_reinit = []
+        self.last_popup_time = datetime.now()
 
         configfile = os.path.normpath(os.path.expanduser(args.configfile))
 
@@ -118,12 +123,16 @@ class KlipperScreen(Gtk.Window):
 
         self.connect("key-press-event", self._key_press_event)
         self.connect("configure_event", self.update_size)
-        monitor = Gdk.Display.get_default().get_primary_monitor()
-        if monitor is None:
-            self.wayland = True
-            monitor = Gdk.Display.get_default().get_monitor(0)
-        if monitor is None:
-            raise RuntimeError("Couldn't get default monitor")
+        monitor_amount = Gdk.Display.get_n_monitors(Gdk.Display.get_default())
+        try:
+            mon_n = int(args.monitor)
+            if not (-1 < mon_n < monitor_amount):
+                raise ValueError
+        except ValueError:
+            mon_n = 0
+        logging.info(f"Monitors: {monitor_amount} using number: {mon_n}")
+        monitor = Gdk.Display.get_default().get_monitor(mon_n)
+        self.wayland = Gdk.Display.get_default().get_primary_monitor() is None
         self.width = self._config.get_main_config().getint("width", None)
         self.height = self._config.get_main_config().getint("height", None)
         if 'XDG_CURRENT_DESKTOP' in os.environ:
@@ -134,12 +143,14 @@ class KlipperScreen(Gtk.Window):
                 self.height = max(int(monitor.get_geometry().height * .5), 320)
         if self.width or self.height:
             logging.info("Setting windowed mode")
+            if mon_n > 0:
+                logging.error("Monitor selection is only supported for fullscreen")
             self.set_resizable(True)
             self.windowed = True
         else:
             self.width = monitor.get_geometry().width
             self.height = monitor.get_geometry().height
-            self.fullscreen()
+            self.fullscreen_on_monitor(self.get_screen(), mon_n)
         self.set_default_size(self.width, self.height)
         self.aspect_ratio = self.width / self.height
         self.vertical_mode = self.aspect_ratio < 1.0
@@ -169,6 +180,11 @@ class KlipperScreen(Gtk.Window):
         self.set_screenblanking_timeout(self._config.get_main_config().get('screen_blanking'))
         self.log_notification("KlipperScreen Started", 1)
         self.initial_connection()
+        if sys.version_info == (3, 7):
+            GLib.timeout_add_seconds(2, self.show_popup_message,
+                                     _("Warning") + f" Python 3.7\n"
+                                     + _("Ended official support in June 2023") + "\n"
+                                     + _("KlipperScreen will drop support in June 2024"), 2)
 
     def initial_connection(self):
         self.printers = self._config.get_printers()
@@ -182,7 +198,7 @@ class KlipperScreen(Gtk.Window):
             "shutdown": self.state_shutdown
         }
         for printer in self.printers:
-            printer["data"] = Printer(state_execute, state_callbacks, self.process_busy_state)
+            printer["data"] = Printer(state_execute, state_callbacks)
         default_printer = self._config.get_main_config().get('default_printer')
         logging.debug(f"Default printer: {default_printer}")
         if [True for p in self.printers if default_printer in p]:
@@ -196,24 +212,25 @@ class KlipperScreen(Gtk.Window):
 
     def connect_printer(self, name):
         self.connecting_to_printer = name
+        if self.files:
+            self.files.__init__(self)
+        gc.collect()
         if self._ws is not None and self._ws.connected:
             self._ws.close()
             self.connected_printer = None
             self.printer.state = "disconnected"
-            if self.files:
-                self.files.reset()
-                self.files = None
-
         self.connecting = True
         self.initialized = False
 
-        ind = 0
         logging.info(f"Connecting to printer: {name}")
-        for printer in self.printers:
-            if name == list(printer)[0]:
-                ind = self.printers.index(printer)
-                break
-
+        ind = next(
+            (
+                self.printers.index(printer)
+                for printer in self.printers
+                if name == list(printer)[0]
+            ),
+            0,
+        )
         self.printer = self.printers[ind]["data"]
         self.apiclient = KlippyRest(
             self.printers[ind][name]["moonraker_host"],
@@ -232,8 +249,8 @@ class KlipperScreen(Gtk.Window):
                                    self.printers[ind][name]["moonraker_host"],
                                    self.printers[ind][name]["moonraker_port"],
                                    )
-
-        self.files = KlippyFiles(self)
+        if self.files is None:
+            self.files = KlippyFiles(self)
         self._ws.initial_connect()
 
     def ws_subscribe(self):
@@ -256,6 +273,7 @@ class KlipperScreen(Gtk.Window):
                 "motion_report": ["live_position", "live_velocity", "live_extruder_velocity"],
                 "exclude_object": ["current_object", "objects", "excluded_objects"],
                 "manual_probe": ['is_active'],
+                "screws_tilt_adjust": ['results', 'error'],
             }
         }
         for extruder in self.printer.get_tools():
@@ -263,6 +281,10 @@ class KlipperScreen(Gtk.Window):
                 "target", "temperature", "pressure_advance", "smooth_time", "power"]
         for h in self.printer.get_heaters():
             requested_updates['objects'][h] = ["target", "temperature", "power"]
+        for t in self.printer.get_temp_sensors():
+            requested_updates['objects'][t] = ["temperature"]
+        for f in self.printer.get_temp_fans():
+            requested_updates['objects'][f] = ["target", "temperature"]
         for f in self.printer.get_fans():
             requested_updates['objects'][f] = ["speed"]
         for f in self.printer.get_filament_sensors():
@@ -312,7 +334,6 @@ class KlipperScreen(Gtk.Window):
         logging.debug(f"Current panel hierarchy: {' > '.join(self._cur_panels)}")
         if hasattr(self.panels[panel], "process_update"):
             self.process_update("notify_status_update", self.printer.data)
-            self.process_update("notify_busy", self.printer.busy)
         if hasattr(self.panels[panel], "activate"):
             self.panels[panel].activate()
         self.show_all()
@@ -326,18 +347,21 @@ class KlipperScreen(Gtk.Window):
         self.process_update("notify_log", log_entry)
 
     def show_popup_message(self, message, level=3):
+        if (datetime.now() - self.last_popup_time).seconds < 1:
+            return
+        self.last_popup_time = datetime.now()
         self.close_screensaver()
         if self.popup_message is not None:
             self.close_popup_message()
 
         self.log_notification(message, level)
 
-        msg = Gtk.Button(label=f"{message}")
-        msg.set_hexpand(True)
-        msg.set_vexpand(True)
-        msg.get_child().set_line_wrap(True)
-        msg.get_child().set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
-        msg.get_child().set_max_width_chars(40)
+        msg = Gtk.Button(label=f"{message}", hexpand=True, vexpand=True)
+        for widget in msg.get_children():
+            if isinstance(widget, Gtk.Label):
+                widget.set_line_wrap(True)
+                widget.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
+                widget.set_max_width_chars(40)
         msg.connect("clicked", self.close_popup_message)
         msg.get_style_context().add_class("message_popup")
         if level == 1:
@@ -350,10 +374,9 @@ class KlipperScreen(Gtk.Window):
             msg.get_style_context().add_class("message_popup_error")
             logging.info(f'error: {message}')
 
-        popup = Gtk.Popover.new(self.base_panel.titlebar)
+        popup = Gtk.Popover(relative_to=self.base_panel.titlebar,
+                            halign=Gtk.Align.CENTER, width_request=int(self.width * .9))
         popup.get_style_context().add_class("message_popup_popover")
-        popup.set_size_request(self.width * .9, -1)
-        popup.set_halign(Gtk.Align.CENTER)
         popup.add(msg)
         popup.popup()
 
@@ -364,7 +387,8 @@ class KlipperScreen(Gtk.Window):
             if self.popup_timeout is not None:
                 GLib.source_remove(self.popup_timeout)
                 self.popup_timeout = None
-            self.popup_timeout = GLib.timeout_add_seconds(10, self.close_popup_message)
+            timeout = 300 if level == 2 else 10
+            self.popup_timeout = GLib.timeout_add_seconds(timeout, self.close_popup_message)
 
         return False
 
@@ -380,24 +404,13 @@ class KlipperScreen(Gtk.Window):
     def show_error_modal(self, err, e=""):
         logging.error(f"Showing error modal: {err} {e}")
 
-        title = Gtk.Label()
+        title = Gtk.Label(wrap=True, wrap_mode=Pango.WrapMode.CHAR, hexpand=True, halign=Gtk.Align.START)
         title.set_markup(f"<b>{err}</b>\n")
-        title.set_line_wrap(True)
-        title.set_line_wrap_mode(Pango.WrapMode.CHAR)
-        title.set_halign(Gtk.Align.START)
-        title.set_hexpand(True)
-        version = Gtk.Label(label=f"{functions.get_software_version()}")
-        version.set_halign(Gtk.Align.END)
+        version = Gtk.Label(label=f"{functions.get_software_version()}", halign=Gtk.Align.END)
 
         help_msg = _("Provide KlipperScreen.log when asking for help.\n")
-        message = Gtk.Label(label=f"{help_msg}\n\n{e}")
-        message.set_line_wrap(True)
+        message = Gtk.Label(label=f"{help_msg}\n\n{e}", wrap=True)
         scroll = self.gtk.ScrolledWindow(steppers=False)
-        scroll.set_vexpand(True)
-        if self.vertical_mode:
-            scroll.set_size_request(self.gtk.width - 30, self.gtk.height * .6)
-        else:
-            scroll.set_size_request(self.gtk.width - 30, self.gtk.height * .45)
         scroll.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         scroll.add(message)
 
@@ -419,6 +432,7 @@ class KlipperScreen(Gtk.Window):
     def restart_ks(self, *args):
         logging.debug(f"Restarting {sys.executable} {' '.join(sys.argv)}")
         os.execv(sys.executable, ['python'] + sys.argv)
+        # noinspection PyUnreachableCode
         self._ws.send_method("machine.services.restart", {"service": "KlipperScreen"})  # Fallback
 
     def init_style(self):
@@ -546,10 +560,8 @@ class KlipperScreen(Gtk.Window):
         close = Gtk.Button()
         close.connect("clicked", self.close_screensaver)
 
-        box = Gtk.Box()
-        box.set_size_request(self.width, self.height)
+        box = Gtk.Box(halign=Gtk.Align.CENTER, width_request=self.width, height_request=self.height)
         box.pack_start(close, True, True, 0)
-        box.set_halign(Gtk.Align.CENTER)
         box.get_style_context().add_class("screensaver")
         self.remove(self.base_panel.main_grid)
         self.add(box)
@@ -647,22 +659,18 @@ class KlipperScreen(Gtk.Window):
         self.base_panel.show_heaters(False)
         self.show_panel("printer_select", _("Printer Select"), remove_all=True)
 
-    def process_busy_state(self, busy):
-        self.process_update("notify_busy", busy)
-        return False
-
     def websocket_disconnected(self, msg):
+        logging.debug("### websocket_disconnected")
         self.printer_initializing(msg, remove=True)
         self.printer.state = "disconnected"
         self.connecting = True
         self.connected_printer = None
-        self.files.reset()
-        self.files = None
         self.initialized = False
         self.connect_printer(self.connecting_to_printer)
 
     def state_disconnected(self):
         logging.debug("### Going to disconnected")
+        self.printer.stop_tempstore_updates()
         self.close_screensaver()
         self.initialized = False
         self.reinit_count = 0
@@ -704,6 +712,7 @@ class KlipperScreen(Gtk.Window):
 
     def state_shutdown(self):
         self.close_screensaver()
+        self.printer.stop_tempstore_updates()
         msg = self.printer.get_stat("webhooks", "state_message")
         msg = msg if "ready" not in msg else ""
         self.printer_initializing(_("Klipper has shutdown") + "\n\n" + msg, remove=True)
@@ -743,20 +752,27 @@ class KlipperScreen(Gtk.Window):
             return
         elif action == "notify_klippy_shutdown":
             self.printer.process_update({'webhooks': {'state': "shutdown"}})
+            return
         elif action == "notify_klippy_ready":
             if not self.initialized:
-                logging.debug("Still not initialized")
+                self.reinit_count = 0
+                self._init_printer("Reconnecting", klipper=True)
                 return
             self.printer.process_update({'webhooks': {'state': "ready"}})
+            return
         elif action == "notify_status_update" and self.printer.state != "shutdown":
             self.printer.process_update(data)
             if 'manual_probe' in data and data['manual_probe']['is_active'] and 'zcalibrate' not in self._cur_panels:
                 self.show_panel("zcalibrate", _('Z Calibrate'))
+            if "screws_tilt_adjust" in data and 'bed_level' not in self._cur_panels:
+                self.show_panel("bed_level", _('Bed Level'))
         elif action == "notify_filelist_changed":
             if self.files is not None:
                 self.files.process_update(data)
+            return
         elif action == "notify_metadata_update":
             self.files.request_metadata(data['filename'])
+            return
         elif action == "notify_update_response":
             if 'message' in data and 'Error' in data['message']:
                 logging.error(f"{action}:{data['message']}")
@@ -769,7 +785,16 @@ class KlipperScreen(Gtk.Window):
             self.panels['splash_screen'].check_power_status()
         elif action == "notify_gcode_response" and self.printer.state not in ["error", "shutdown"]:
             if not (data.startswith("B:") or data.startswith("T:")):
-                if data.startswith("echo: "):
+                if data.startswith("// action:"):
+                    action = data[10:]
+                    if action.startswith('prompt_begin'):
+                        if self.prompt is not None:
+                            self.prompt.end()
+                        self.prompt = Prompt(self)
+                    if self.prompt is None:
+                        return
+                    self.prompt.decode(action)
+                elif data.startswith("echo: "):
                     self.show_popup_message(data[6:], 1)
                 elif data.startswith("!! "):
                     self.show_popup_message(data[3:], 3)
@@ -793,8 +818,8 @@ class KlipperScreen(Gtk.Window):
 
     def _confirm_send_action(self, widget, text, method, params=None):
         buttons = [
-            {"name": _("Accept"), "response": Gtk.ResponseType.OK},
-            {"name": _("Cancel"), "response": Gtk.ResponseType.CANCEL}
+            {"name": _("Accept"), "response": Gtk.ResponseType.OK, "style": 'dialog-info'},
+            {"name": _("Cancel"), "response": Gtk.ResponseType.CANCEL, "style": 'dialog-error'}
         ]
 
         try:
@@ -803,14 +828,9 @@ class KlipperScreen(Gtk.Window):
         except Exception as e:
             logging.debug(f"Error parsing jinja for confirm_send_action\n{e}\n\n{traceback.format_exc()}")
 
-        label = Gtk.Label()
+        label = Gtk.Label(hexpand=True, vexpand=True, halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER,
+                          wrap=True, wrap_mode=Pango.WrapMode.WORD_CHAR)
         label.set_markup(text)
-        label.set_hexpand(True)
-        label.set_halign(Gtk.Align.CENTER)
-        label.set_vexpand(True)
-        label.set_valign(Gtk.Align.CENTER)
-        label.set_line_wrap(True)
-        label.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
 
         if self.confirm is not None:
             self.gtk.remove_dialog(self.confirm)
@@ -822,8 +842,6 @@ class KlipperScreen(Gtk.Window):
         self.gtk.remove_dialog(dialog)
         if response_id == Gtk.ResponseType.OK:
             self._send_action(None, method, params)
-        if method == "server.files.delete_directory":
-            GLib.timeout_add_seconds(2, self.files.refresh_files)
 
     def _send_action(self, widget, method, params):
         logging.info(f"{method}: {params}")
@@ -863,17 +881,21 @@ class KlipperScreen(Gtk.Window):
             else:
                 self._ws.klippy.power_device_off(dev)
 
-    def _init_printer(self, msg, remove=False):
+    def _init_printer(self, msg, remove=False, klipper=False):
         self.printer_initializing(msg, remove)
         self.initializing = False
-        GLib.timeout_add_seconds(3, self.init_printer)
-        return False
+        if klipper:
+            GLib.timeout_add_seconds(3, self.init_klipper)
+        else:
+            GLib.timeout_add_seconds(3, self.init_printer)
 
     def init_printer(self):
         if self.initializing:
+            logging.info("Already Initializing")
             return False
         self.initializing = True
         if self.reinit_count > self.max_retries or 'printer_select' in self._cur_panels:
+            logging.info("Stopping Retries")
             self.initializing = False
             return False
         state = self.apiclient.get_server_info()
@@ -885,11 +907,11 @@ class KlipperScreen(Gtk.Window):
         self.connected_printer = self.connecting_to_printer
         self.base_panel.set_ks_printer_cfg(self.connected_printer)
 
+        self.init_server(state["result"])
         # Moonraker is ready, set a loop to init the printer
-        self.reinit_count += 1
+        return self.init_klipper(state["result"])
 
-        server_info = self.apiclient.get_server_info()["result"]
-        logging.info(f"Moonraker info {server_info}")
+    def init_server(self, server_info):
         popup = ''
         level = 2
         if server_info["warnings"]:
@@ -919,13 +941,23 @@ class KlipperScreen(Gtk.Window):
         if "spoolman" in server_info["components"]:
             self.printer.enable_spoolman()
 
-        if state['result']['klippy_connected'] is False:
+    def init_klipper(self, server_info=None):
+        if self.reinit_count > self.max_retries or 'printer_select' in self._cur_panels:
+            logging.info("Stopping Retries")
+            return False
+        if not server_info:
+            server_info = self.apiclient.get_server_info()["result"]
+        logging.info(f"Moonraker info {server_info}")
+
+        self.reinit_count += 1
+
+        if server_info['klippy_connected'] is False:
             logging.info("Klipper not connected")
             msg = _("Moonraker: connected") + "\n\n"
-            msg += f"Klipper: {state['result']['klippy_state']}" + "\n\n"
+            msg += f"Klipper: {server_info['klippy_state']}" + "\n\n"
             if self.reinit_count <= self.max_retries:
                 msg += _("Retrying") + f' #{self.reinit_count}'
-            return self._init_printer(msg)
+            return self._init_printer(msg, klipper=True)
         printer_info = self.apiclient.get_printer_info()
         if printer_info is False:
             return self._init_printer("Unable to get printer info from moonraker")
@@ -940,7 +972,9 @@ class KlipperScreen(Gtk.Window):
         self.ws_subscribe()
         extra_items = (self.printer.get_tools()
                        + self.printer.get_heaters()
+                       + self.printer.get_temp_sensors()
                        + self.printer.get_fans()
+                       + self.printer.get_temp_fans()
                        + self.printer.get_filament_sensors()
                        + self.printer.get_output_pins()
                        + self.printer.get_leds()
@@ -950,10 +984,8 @@ class KlipperScreen(Gtk.Window):
                                                                                extra_items))
         if data is False:
             return self._init_printer("Error getting printer object data with extra items")
-        if len(self.printer.get_temp_devices()) > 0:
-            self.init_tempstore()
 
-        self.files.initialize()
+        self.files.set_gcodes_path()
         self.files.refresh_files()
 
         logging.info("Printer initialized")
@@ -965,6 +997,8 @@ class KlipperScreen(Gtk.Window):
         return False
 
     def init_tempstore(self):
+        if len(self.printer.get_temp_devices()) == 0:
+            return
         tempstore = self.apiclient.send_request("server/temperature_store")
         if tempstore and 'result' in tempstore and tempstore['result']:
             self.printer.init_temp_store(tempstore['result'])
@@ -1070,7 +1104,8 @@ def main():
     homedir = os.path.expanduser("~")
 
     parser.add_argument(
-        "-c", "--configfile", default=os.path.join(homedir, "KlipperScreen.conf"), metavar='<configfile>',
+        "-c", "--configfile",
+        default="", metavar='<configfile>',
         help="Location of KlipperScreen configuration file"
     )
     logdir = os.path.join(homedir, "printer_data", "logs")
@@ -1080,6 +1115,10 @@ def main():
         "-l", "--logfile", default=os.path.join(logdir, "KlipperScreen.log"), metavar='<logfile>',
         help="Location of KlipperScreen logfile output"
     )
+    parser.add_argument(
+        "-m", "--monitor", default="0", metavar='<monitor>',
+        help="Number of the monitor, that will show Klipperscreen (default: 0)"
+    )
     args = parser.parse_args()
 
     functions.setup_logging(os.path.normpath(os.path.expanduser(args.logfile)))
